
#include <Arduino_AdvancedAnalog.h>
#define BUFSIZE 8
#define NBUF 4
uint32_t sample_rate = 50000;
double sample_period;
AdvancedDAC dac0(A12);
AdvancedDAC dac1(A13);
double deuxpi = 2*PI;

#define NECHANT 256
#define SHIFT_ACCUM 24
uint16_t table0[NECHANT];
uint16_t table1[NECHANT];
uint32_t accum0,accum1;
uint32_t increm0,increm1;

// transformation nombre -> tension
float offset = 2048;
float num2volt = 3.217/4096;
float volt2num = 4096/3.217;

// définition des signaux
#define CORRECT 1.00014
float f0 = 100.0/CORRECT;
float f1 = 100.0/CORRECT;
float noise0 = 0.05;
float noise1 = 0.05;
int16_t randNoise;

float signal0(float phi) {
  return 0.5*sin(deuxpi*phi)+0.2*sin(2*deuxpi*phi)+0.1*sin(4*deuxpi*phi+PI/3);
}

float signal1(float phi) {
  return 0.5*sin(deuxpi*phi);
}

void setup() {
  pinMode(13,OUTPUT);
  
  sample_period = 1.0/sample_rate; 
  Serial.begin(115200);
  while(!Serial);
for (uint16_t i=0; i<NECHANT; i++) {
    table0[i] = signal0(1.0*i/NECHANT)*volt2num + offset;
    table1[i] = signal1(1.0*i/NECHANT)*volt2num + offset;
  }
  increm0 = (uint32_t) (((float)(0xFFFFFFFF))*((float)(f0)*1.0/sample_rate)); // incrément de l'accumulateur de phase
  increm1 = (uint32_t) (((float)(0xFFFFFFFF))*((float)(f1)*1.0/sample_rate));
  accum0 = accum1 = 0;
  RCC->AHB2ENR |= RCC_AHB2ENR_RNGEN;
  RNG->CR |= RNG_CR_RNGEN;

  if (!dac0.begin(AN_RESOLUTION_12, sample_rate,BUFSIZE, NBUF)) {
    Serial.println("Echec de démarrage de DAC0");
      while (1);
  }
  if (!dac1.begin(AN_RESOLUTION_12, sample_rate,BUFSIZE, NBUF)) {
    Serial.println("Echec de démarrage de DAC1");
      while (1);
  }
  
}

void loop() {
  if (dac0.available()&&dac1.available()) {
      GPIOH->BSRR |= GPIO_BSRR_BS6; //digitalWrite(13,HIGH);
      SampleBuffer buf_dac0 = dac0.dequeue();
      uint16_t *buf_dac0_data = buf_dac0.data();
      SampleBuffer buf_dac1 = dac1.dequeue();
      uint16_t *buf_dac1_data = buf_dac1.data();
      
      for (uint16_t i=0; i<BUFSIZE; i++) {
          accum0 += increm0;
          accum1 += increm1;
          if (RNG->SR & RNG_SR_DRDY) randNoise = volt2num*(((double)(RNG->DR-0x7FFFFFFF))/0x7FFFFFFF)*noise0;
          buf_dac0_data[i] = table0[accum0 >> SHIFT_ACCUM]+randNoise;
          if (RNG->SR & RNG_SR_DRDY) randNoise = volt2num*(((double)(RNG->DR-0x7FFFFFFF))/0x7FFFFFFF)*noise1;
          buf_dac1_data[i] = table1[accum1 >> SHIFT_ACCUM]+randNoise;
      }
      dac0.write(buf_dac0);
      dac1.write(buf_dac1);
      GPIOH->BSRR |= GPIO_BSRR_BR6; //digitalWrite(13,LOW);
  }
  
}
   
                   